@@ -9,7 +9,7 @@ namespace ts.server {
         newLine: "\n",
         useCaseSensitiveFileNames: true,
         write(s): void { lastWrittenToHost = s; },
-        readFile(): string { return void 0; },
+        readFile: () => undefined,
         writeFile: noop,
         resolvePath(): string { return void 0; },
         fileExists: () => false,
@@ -19,25 +19,13 @@ namespace ts.server {
         getExecutingFilePath(): string { return void 0; },
         getCurrentDirectory(): string { return void 0; },
         getEnvironmentVariable(): string { return ""; },
-        readDirectory(): string[] { return []; },
+        readDirectory() { return []; },
         exit: noop,
         setTimeout() { return 0; },
         clearTimeout: noop,
         setImmediate: () => 0,
         clearImmediate: noop,
-        createHash: s => s
-    };
-    const nullCancellationToken: HostCancellationToken = { isCancellationRequested: () => false };
-    const mockLogger: Logger = {
-        close: noop,
-        hasLevel(): boolean { return false; },
-        loggingEnabled(): boolean { return false; },
-        perftrc: noop,
-        info: noop,
-        startGroup: noop,
-        endGroup: noop,
-        msg: noop,
-        getLogFileName: (): string => undefined
+        createHash: Harness.LanguageService.mockHash,
     };
 
     class TestSession extends Session {
@@ -50,8 +38,23 @@ namespace ts.server {
         let session: TestSession;
         let lastSent: protocol.Message;
 
+        function createSession(): TestSession {
+            const opts: server.SessionOptions = {
+                host: mockHost,
+                cancellationToken: nullCancellationToken,
+                useSingleInferredProject: false,
+                useInferredProjectPerProjectRoot: false,
+                typingsInstaller: undefined,
+                byteLength: Utils.byteLength,
+                hrtime: process.hrtime,
+                logger: projectSystem.nullLogger,
+                canUseEvents: true
+            };
+            return new TestSession(opts);
+        }
+
         beforeEach(() => {
-            session = new TestSession(mockHost, nullCancellationToken, /*useOneInferredProject*/ false, /*typingsInstaller*/ undefined, Utils.byteLength, process.hrtime, mockLogger, /*canUseEvents*/ true);
+            session = createSession();
             session.send = (msg: protocol.Message) => {
                 lastSent = msg;
             };
@@ -79,14 +82,15 @@ namespace ts.server {
 
                 session.executeCommand(req);
 
-                expect(lastSent).to.deep.equal(<protocol.Response>{
+                const expected: protocol.Response = {
                     command: CommandNames.Unknown,
                     type: "response",
                     seq: 0,
                     message: "Unrecognized JSON command: foobar",
                     request_seq: 0,
                     success: false
-                });
+                };
+                expect(lastSent).to.deep.equal(expected);
             });
             it("should return a tuple containing the response and if a response is required on success", () => {
                 const req: protocol.ConfigureRequest = {
@@ -120,7 +124,7 @@ namespace ts.server {
                     type: "request",
                     arguments: {
                         formatOptions: {
-                            indentStyle: "Block"
+                            indentStyle: protocol.IndentStyle.Block,
                         }
                     }
                 };
@@ -135,11 +139,11 @@ namespace ts.server {
                     type: "request",
                     arguments: {
                         options: {
-                            module: "System",
-                            target: "ES5",
-                            jsx: "React",
-                            newLine: "Lf",
-                            moduleResolution: "Node"
+                            module: protocol.ModuleKind.System,
+                            target: protocol.ScriptTarget.ES5,
+                            jsx: protocol.JsxEmit.React,
+                            newLine: protocol.NewLineKind.Lf,
+                            moduleResolution: protocol.ModuleResolutionKind.Node,
                         }
                     }
                 };
@@ -158,12 +162,81 @@ namespace ts.server {
         });
 
         describe("onMessage", () => {
+            const allCommandNames: CommandNames[] = [
+                CommandNames.Brace,
+                CommandNames.BraceFull,
+                CommandNames.BraceCompletion,
+                CommandNames.Change,
+                CommandNames.Close,
+                CommandNames.Completions,
+                CommandNames.CompletionsFull,
+                CommandNames.CompletionDetails,
+                CommandNames.CompileOnSaveAffectedFileList,
+                CommandNames.Configure,
+                CommandNames.Definition,
+                CommandNames.DefinitionFull,
+                CommandNames.Implementation,
+                CommandNames.ImplementationFull,
+                CommandNames.Exit,
+                CommandNames.Format,
+                CommandNames.Formatonkey,
+                CommandNames.FormatFull,
+                CommandNames.FormatonkeyFull,
+                CommandNames.FormatRangeFull,
+                CommandNames.Geterr,
+                CommandNames.GeterrForProject,
+                CommandNames.SemanticDiagnosticsSync,
+                CommandNames.SyntacticDiagnosticsSync,
+                CommandNames.NavBar,
+                CommandNames.NavBarFull,
+                CommandNames.Navto,
+                CommandNames.NavtoFull,
+                CommandNames.NavTree,
+                CommandNames.NavTreeFull,
+                CommandNames.Occurrences,
+                CommandNames.DocumentHighlights,
+                CommandNames.DocumentHighlightsFull,
+                CommandNames.Open,
+                CommandNames.Quickinfo,
+                CommandNames.QuickinfoFull,
+                CommandNames.References,
+                CommandNames.ReferencesFull,
+                CommandNames.Reload,
+                CommandNames.Rename,
+                CommandNames.RenameInfoFull,
+                CommandNames.RenameLocationsFull,
+                CommandNames.Saveto,
+                CommandNames.SignatureHelp,
+                CommandNames.SignatureHelpFull,
+                CommandNames.TypeDefinition,
+                CommandNames.ProjectInfo,
+                CommandNames.ReloadProjects,
+                CommandNames.Unknown,
+                CommandNames.OpenExternalProject,
+                CommandNames.CloseExternalProject,
+                CommandNames.SynchronizeProjectList,
+                CommandNames.ApplyChangedToOpenFiles,
+                CommandNames.EncodedSemanticClassificationsFull,
+                CommandNames.Cleanup,
+                CommandNames.OutliningSpans,
+                CommandNames.TodoComments,
+                CommandNames.Indentation,
+                CommandNames.DocCommentTemplate,
+                CommandNames.CompilerOptionsDiagnosticsFull,
+                CommandNames.NameOrDottedNameSpan,
+                CommandNames.BreakpointStatement,
+                CommandNames.CompilerOptionsForInferredProjects,
+                CommandNames.GetCodeFixes,
+                CommandNames.GetCodeFixesFull,
+                CommandNames.GetSupportedCodeFixes,
+                CommandNames.GetApplicableRefactors,
+                CommandNames.GetEditsForRefactor,
+                CommandNames.GetEditsForRefactorFull,
+            ];
+
             it("should not throw when commands are executed with invalid arguments", () => {
                 let i = 0;
-                for (const name in CommandNames) {
-                    if (!Object.prototype.hasOwnProperty.call(CommandNames, name)) {
-                        continue;
-                    }
+                for (const name of allCommandNames) {
                     const req: protocol.Request = {
                         command: name,
                         seq: i,
@@ -306,7 +379,7 @@ namespace ts.server {
                     request_seq: 0,
                     type: "response",
                     command,
-                    body: body,
+                    body,
                     success: true
                 });
             });
@@ -318,7 +391,17 @@ namespace ts.server {
             lastSent: protocol.Message;
             customHandler = "testhandler";
             constructor() {
-                super(mockHost, nullCancellationToken, /*useOneInferredProject*/ false, /*typingsInstaller*/ undefined, Utils.byteLength, process.hrtime, mockLogger, /*canUseEvents*/ true);
+                super({
+                    host: mockHost,
+                    cancellationToken: nullCancellationToken,
+                    useSingleInferredProject: false,
+                    useInferredProjectPerProjectRoot: false,
+                    typingsInstaller: undefined,
+                    byteLength: Utils.byteLength,
+                    hrtime: process.hrtime,
+                    logger: projectSystem.nullLogger,
+                    canUseEvents: true
+                });
                 this.addProtocolHandler(this.customHandler, () => {
                     return { response: undefined, responseRequired: true };
                 });
@@ -326,7 +409,7 @@ namespace ts.server {
             send(msg: protocol.Message) {
                 this.lastSent = msg;
             }
-        };
+        }
 
         it("can override methods such as send", () => {
             const session = new TestSession();
@@ -344,7 +427,7 @@ namespace ts.server {
                 request_seq: 0,
                 type: "response",
                 command,
-                body: body,
+                body,
                 success: true
             });
         });
@@ -367,7 +450,7 @@ namespace ts.server {
                     assert(this.projectService);
                     expect(this.projectService).to.be.instanceOf(ProjectService);
                 }
-            };
+            }
             new ServiceSession();
         });
     });
@@ -376,7 +459,17 @@ namespace ts.server {
         class InProcSession extends Session {
             private queue: protocol.Request[] = [];
             constructor(private client: InProcClient) {
-                super(mockHost, nullCancellationToken, /*useOneInferredProject*/ false, /*typingsInstaller*/ undefined, Utils.byteLength, process.hrtime, mockLogger, /*canUseEvents*/ true);
+                super({
+                    host: mockHost,
+                    cancellationToken: nullCancellationToken,
+                    useSingleInferredProject: false,
+                    useInferredProjectPerProjectRoot: false,
+                    typingsInstaller: undefined,
+                    byteLength: Utils.byteLength,
+                    hrtime: process.hrtime,
+                    logger: projectSystem.nullLogger,
+                    canUseEvents: true
+                });
                 this.addProtocolHandler("echo", (req: protocol.Request) => ({
                     response: req.arguments,
                     responseRequired: true
@@ -416,14 +509,15 @@ namespace ts.server {
         class InProcClient {
             private server: InProcSession;
             private seq = 0;
-            private callbacks = createMap<(resp: protocol.Response) => void>();
+            private callbacks: Array<(resp: protocol.Response) => void> = [];
             private eventHandlers = createMap<(args: any) => void>();
 
             handle(msg: protocol.Message): void {
                 if (msg.type === "response") {
                     const response = <protocol.Response>msg;
-                    if (response.request_seq in this.callbacks) {
-                        this.callbacks[response.request_seq](response);
+                    const handler = this.callbacks[response.request_seq];
+                    if (handler) {
+                        handler(response);
                         delete this.callbacks[response.request_seq];
                     }
                 }
@@ -434,13 +528,14 @@ namespace ts.server {
             }
 
             emit(name: string, args: any): void {
-                if (name in this.eventHandlers) {
-                    this.eventHandlers[name](args);
+                const handler = this.eventHandlers.get(name);
+                if (handler) {
+                    handler(args);
                 }
             }
 
             on(name: string, handler: (args: any) => void): void {
-                this.eventHandlers[name] = handler;
+                this.eventHandlers.set(name, handler);
             }
 
             connect(session: InProcSession): void {
@@ -460,7 +555,7 @@ namespace ts.server {
                 });
                 this.callbacks[this.seq] = callback;
             }
-        };
+        }
 
         it("can be constructed and respond to commands", (done) => {
             const cli = new InProcClient();
