@@ -16,7 +16,7 @@ namespace ts.server {
 
         public getVersion() {
             return this.svc
-                ? `SVC-${this.svcVersion}-${this.svc.getSnapshot().version}`
+                ? `SVC-${this.svcVersion}-${this.svc.getSnapshotVersion()}`
                 : `Text-${this.textVersion}`;
         }
 
@@ -48,7 +48,7 @@ namespace ts.server {
 
         public reloadFromFile(tempFileName?: string) {
             if (this.svc || (tempFileName !== this.fileName)) {
-                this.reload(this.getFileText(tempFileName))
+                this.reload(this.getFileText(tempFileName));
             }
             else {
                 this.setText(undefined);
@@ -61,30 +61,20 @@ namespace ts.server {
                 : ScriptSnapshot.fromString(this.getOrLoadText());
         }
 
-        public getLineInfo(line: number) {
-            return this.switchToScriptVersionCache().getSnapshot().index.lineNumberToInfo(line);
+        public getLineInfo(line: number): AbsolutePositionAndLineText {
+            return this.switchToScriptVersionCache().getLineInfo(line);
         }
         /**
          *  @param line 0 based index
          */
-        lineToTextSpan(line: number) {
+        lineToTextSpan(line: number): TextSpan {
             if (!this.svc) {
                 const lineMap = this.getLineMap();
                 const start = lineMap[line]; // -1 since line is 1-based
                 const end = line + 1 < lineMap.length ? lineMap[line + 1] : this.text.length;
-                return ts.createTextSpanFromBounds(start, end);
+                return createTextSpanFromBounds(start, end);
             }
-            const index = this.svc.getSnapshot().index;
-            const lineInfo = index.lineNumberToInfo(line + 1);
-            let len: number;
-            if (lineInfo.leaf) {
-                len = lineInfo.leaf.text.length;
-            }
-            else {
-                const nextLineInfo = index.lineNumberToInfo(line + 2);
-                len = nextLineInfo.offset - lineInfo.offset;
-            }
-            return ts.createTextSpan(lineInfo.offset, len);
+            return this.svc.lineToTextSpan(line);
         }
 
         /**
@@ -93,27 +83,19 @@ namespace ts.server {
          */
         lineOffsetToPosition(line: number, offset: number): number {
             if (!this.svc) {
-                return computePositionOfLineAndCharacter(this.getLineMap(), line - 1, offset - 1);
+                return computePositionOfLineAndCharacter(this.getLineMap(), line - 1, offset - 1, this.text);
             }
-            const index = this.svc.getSnapshot().index;
 
-            const lineInfo = index.lineNumberToInfo(line);
             // TODO: assert this offset is actually on the line
-            return (lineInfo.offset + offset - 1);
+            return this.svc.lineOffsetToPosition(line, offset);
         }
 
-        /**
-         * @param line 1-based index
-         * @param offset 1-based index
-         */
-        positionToLineOffset(position: number): ILineInfo {
+        positionToLineOffset(position: number): protocol.Location {
             if (!this.svc) {
                 const { line, character } = computeLineAndCharacterOfPosition(this.getLineMap(), position);
                 return { line: line + 1, offset: character + 1 };
             }
-            const index = this.svc.getSnapshot().index;
-            const lineOffset = index.charOffsetToLineNumberAndPos(position);
-            return { line: lineOffset.line, offset: lineOffset.offset + 1 };
+            return this.svc.positionToLineOffset(position);
         }
 
         private getFileText(tempFileName?: string) {
@@ -126,7 +108,7 @@ namespace ts.server {
 
         private switchToScriptVersionCache(newText?: string): ScriptVersionCache {
             if (!this.svc) {
-                this.svc = ScriptVersionCache.fromString(this.host, newText !== undefined ? newText : this.getOrLoadText());
+                this.svc = ScriptVersionCache.fromString(newText !== undefined ? newText : this.getOrLoadText());
                 this.svcVersion++;
                 this.text = undefined;
             }
@@ -162,7 +144,7 @@ namespace ts.server {
          * All projects that include this file
          */
         readonly containingProjects: Project[] = [];
-        private formatCodeSettings: ts.FormatCodeSettings;
+        private formatCodeSettings: FormatCodeSettings;
         readonly path: Path;
 
         private fileWatcher: FileWatcher;
@@ -247,7 +229,7 @@ namespace ts.server {
                     }
                     break;
                 default:
-                    removeItemFromSet(this.containingProjects, project);
+                    unorderedRemoveItem(this.containingProjects, project);
                     break;
             }
         }
@@ -257,14 +239,30 @@ namespace ts.server {
                 // detach is unnecessary since we'll clean the list of containing projects anyways
                 p.removeFile(this, /*detachFromProjects*/ false);
             }
-            this.containingProjects.length = 0;
+            clear(this.containingProjects);
         }
 
         getDefaultProject() {
-            if (this.containingProjects.length === 0) {
-                return Errors.ThrowNoProject();
+            switch (this.containingProjects.length) {
+                case 0:
+                    return Errors.ThrowNoProject();
+                case 1:
+                    return this.containingProjects[0];
+                default:
+                    // if this file belongs to multiple projects, the first configured project should be
+                    // the default project; if no configured projects, the first external project should
+                    // be the default project; otherwise the first inferred project should be the default.
+                    let firstExternalProject;
+                    for (const project of this.containingProjects) {
+                        if (project.projectKind === ProjectKind.Configured) {
+                            return project;
+                        }
+                        else if (project.projectKind === ProjectKind.External && !firstExternalProject) {
+                            firstExternalProject = project;
+                        }
+                    }
+                    return firstExternalProject || this.containingProjects[0];
             }
-            return this.containingProjects[0];
         }
 
         registerFileUpdate(): void {
@@ -278,7 +276,7 @@ namespace ts.server {
                 if (!this.formatCodeSettings) {
                     this.formatCodeSettings = getDefaultFormatCodeSettings(this.host);
                 }
-                mergeMaps(this.formatCodeSettings, formatSettings);
+                mergeMapLikes(this.formatCodeSettings, formatSettings);
             }
         }
 
@@ -318,7 +316,7 @@ namespace ts.server {
             }
         }
 
-        getLineInfo(line: number) {
+        getLineInfo(line: number): AbsolutePositionAndLineText {
             return this.textStorage.getLineInfo(line);
         }
 
@@ -348,12 +346,12 @@ namespace ts.server {
             return this.textStorage.lineOffsetToPosition(line, offset);
         }
 
-        /**
-         * @param line 1-based index
-         * @param offset 1-based index
-         */
-        positionToLineOffset(position: number): ILineInfo {
+        positionToLineOffset(position: number): protocol.Location {
             return this.textStorage.positionToLineOffset(position);
         }
+
+        public isJavaScript() {
+            return this.scriptKind === ScriptKind.JS || this.scriptKind === ScriptKind.JSX;
+        }
     }
 }
\ No newline at end of file
