@@ -4,20 +4,19 @@
 namespace ts.formatting {
 
     export class RuleOperationContext {
-        private customContextChecks: { (context: FormattingContext): boolean; }[];
+        private readonly customContextChecks: { (context: FormattingContext): boolean; }[];
 
         constructor(...funcs: { (context: FormattingContext): boolean; }[]) {
             this.customContextChecks = funcs;
         }
 
-        static Any: RuleOperationContext = new RuleOperationContext();
-
+        static readonly Any: RuleOperationContext = new RuleOperationContext();
 
         public IsAny(): boolean {
             return this === RuleOperationContext.Any;
         }
 
-        public  InContext(context: FormattingContext): boolean {
+        public InContext(context: FormattingContext): boolean {
             if (this.IsAny()) {
                 return true;
             }
