@@ -1,6 +1,6 @@
 /* @internal */
 namespace ts.OutliningElementsCollector {
-    export function collectElements(sourceFile: SourceFile): OutliningSpan[] {
+    export function collectElements(sourceFile: SourceFile, cancellationToken: CancellationToken): OutliningSpan[] {
         const elements: OutliningSpan[] = [];
         const collapseText = "...";
 
@@ -8,9 +8,9 @@ namespace ts.OutliningElementsCollector {
             if (hintSpanNode && startElement && endElement) {
                 const span: OutliningSpan = {
                     textSpan: createTextSpanFromBounds(startElement.pos, endElement.end),
-                    hintSpan: createTextSpanFromBounds(hintSpanNode.getStart(), hintSpanNode.end),
+                    hintSpan: createTextSpanFromNode(hintSpanNode, sourceFile),
                     bannerText: collapseText,
-                    autoCollapse: autoCollapse
+                    autoCollapse,
                 };
                 elements.push(span);
             }
@@ -22,7 +22,7 @@ namespace ts.OutliningElementsCollector {
                     textSpan: createTextSpanFromBounds(commentSpan.pos, commentSpan.end),
                     hintSpan: createTextSpanFromBounds(commentSpan.pos, commentSpan.end),
                     bannerText: collapseText,
-                    autoCollapse: autoCollapse
+                    autoCollapse,
                 };
                 elements.push(span);
             }
@@ -38,6 +38,7 @@ namespace ts.OutliningElementsCollector {
                 let singleLineCommentCount = 0;
 
                 for (const currentComment of comments) {
+                    cancellationToken.throwIfCancellationRequested();
 
                     // For single line comments, combine consecutive ones (2 or more) into
                     // a single span from the start of the first till the end of the last
@@ -67,10 +68,10 @@ namespace ts.OutliningElementsCollector {
 
             // Only outline spans of two or more consecutive single line comments
             if (count > 1) {
-                const multipleSingleLineComments = {
+                const multipleSingleLineComments: CommentRange = {
+                    kind: SyntaxKind.SingleLineCommentTrivia,
                     pos: start,
-                    end: end,
-                    kind: SyntaxKind.SingleLineCommentTrivia
+                    end,
                 };
 
                 addOutliningSpanComments(multipleSingleLineComments, /*autoCollapse*/ false);
@@ -84,6 +85,7 @@ namespace ts.OutliningElementsCollector {
         let depth = 0;
         const maxDepth = 20;
         function walk(n: Node): void {
+            cancellationToken.throwIfCancellationRequested();
             if (depth > maxDepth) {
                 return;
             }
@@ -135,7 +137,7 @@ namespace ts.OutliningElementsCollector {
 
                         // Block was a standalone block.  In this case we want to only collapse
                         // the span of the block, independent of any parent span.
-                        const span = createTextSpanFromBounds(n.getStart(), n.end);
+                        const span = createTextSpanFromNode(n);
                         elements.push({
                             textSpan: span,
                             hintSpan: span,
@@ -144,7 +146,7 @@ namespace ts.OutliningElementsCollector {
                         });
                         break;
                     }
-                // Fallthrough.
+                    // falls through
 
                 case SyntaxKind.ModuleBlock: {
                     const openBrace = findChildOfKind(n, SyntaxKind.OpenBraceToken, sourceFile);
