@@ -15,13 +15,13 @@ class c5c { public foo() { } }
 
 //// [augmentedTypesClass3.js]
 // class then module
-var c5 = (function () {
+var c5 = /** @class */ (function () {
     function c5() {
     }
     c5.prototype.foo = function () { };
     return c5;
 }());
-var c5a = (function () {
+var c5a = /** @class */ (function () {
     function c5a() {
     }
     c5a.prototype.foo = function () { };
@@ -30,7 +30,7 @@ var c5a = (function () {
 (function (c5a) {
     var y = 2;
 })(c5a || (c5a = {})); // should be ok
-var c5b = (function () {
+var c5b = /** @class */ (function () {
     function c5b() {
     }
     c5b.prototype.foo = function () { };
@@ -40,7 +40,7 @@ var c5b = (function () {
     c5b.y = 2;
 })(c5b || (c5b = {})); // should be ok
 //// class then import
-var c5c = (function () {
+var c5c = /** @class */ (function () {
     function c5c() {
     }
     c5c.prototype.foo = function () { };
