@@ -7,7 +7,7 @@ function foo2(y = class {[x] = x}, x = 1) {
 
 //// [capturedParametersInInitializers2.js]
 function foo(y, x) {
-    if (y === void 0) { y = (_a = (function () {
+    if (y === void 0) { y = (_a = /** @class */ (function () {
             function class_1() {
             }
             return class_1;
@@ -19,7 +19,7 @@ function foo(y, x) {
     var _a;
 }
 function foo2(y, x) {
-    if (y === void 0) { y = (function () {
+    if (y === void 0) { y = /** @class */ (function () {
         function class_2() {
             this[x] = x;
         }
