@@ -10,11 +10,16 @@ module M {
 }
 
 //// [decoratorOnClassMethod12.js]
-var __extends = (this && this.__extends) || function (d, b) {
-    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
-    function __() { this.constructor = d; }
-    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
-};
+var __extends = (this && this.__extends) || (function () {
+    var extendStatics = Object.setPrototypeOf ||
+        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
+        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
+    return function (d, b) {
+        extendStatics(d, b);
+        function __() { this.constructor = d; }
+        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
+    };
+})();
 var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
     var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
     if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
@@ -23,21 +28,22 @@ var __decorate = (this && this.__decorate) || function (decorators, target, key,
 };
 var M;
 (function (M) {
-    var S = (function () {
+    var _this = this;
+    var S = /** @class */ (function () {
         function S() {
         }
         S.prototype.decorator = function (target, key) { };
         return S;
     }());
-    var C = (function (_super) {
+    var C = /** @class */ (function (_super) {
         __extends(C, _super);
         function C() {
             return _super !== null && _super.apply(this, arguments) || this;
         }
         C.prototype.method = function () { };
+        __decorate([
+            _super.decorator
+        ], C.prototype, "method", null);
         return C;
     }(S));
-    __decorate([
-        _super.decorator
-    ], C.prototype, "method", null);
 })(M || (M = {}));
