@@ -8,9 +8,9 @@ var X = { 0b11: '', 3: '' };
 
 
 //// [duplicateIdentifierDifferentSpelling.js]
-var A = (function () {
+var A = /** @class */ (function () {
     function A() {
-        this[3] = '';
+        this[0b11] = '';
         this[3] = '';
     }
     return A;
