@@ -14,6 +14,6 @@ var x2 = function () { return ; };
 var x3 = 1;
 var x4 = '';
 var x5;
-var x6 = null;
-var x7 = function f() { };
+var x6;
+var x7 = f(), _a = void 0;
 var x8 = /123/;
