@@ -15,7 +15,7 @@ function bar() {
 }
 
 //// [out.js]
-var c = (function () {
+var c = /** @class */ (function () {
     function c() {
     }
     return c;
