@@ -44,7 +44,7 @@ function foo<T, U, V extends Date>(x: T, y: U, z: V) {
 //}
 
 //// [nullAssignableToEveryType.js]
-var C = (function () {
+var C = /** @class */ (function () {
     function C() {
     }
     return C;
