@@ -4,7 +4,7 @@ class C {
 }
 
 //// [restParamModifier.js]
-var C = (function () {
+var C = /** @class */ (function () {
     function C(string) {
         if (string === void 0) { string = []; }
     }
