@@ -60,12 +60,17 @@ else {
 }
 
 //// [typeGuardFunctionOfFormThisErrors.js]
-var __extends = (this && this.__extends) || function (d, b) {
-    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
-    function __() { this.constructor = d; }
-    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
-};
-var RoyalGuard = (function () {
+var __extends = (this && this.__extends) || (function () {
+    var extendStatics = Object.setPrototypeOf ||
+        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
+        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
+    return function (d, b) {
+        extendStatics(d, b);
+        function __() { this.constructor = d; }
+        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
+    };
+})();
+var RoyalGuard = /** @class */ (function () {
     function RoyalGuard() {
     }
     RoyalGuard.prototype.isLeader = function () {
@@ -76,7 +81,7 @@ var RoyalGuard = (function () {
     };
     return RoyalGuard;
 }());
-var LeadGuard = (function (_super) {
+var LeadGuard = /** @class */ (function (_super) {
     __extends(LeadGuard, _super);
     function LeadGuard() {
         return _super !== null && _super.apply(this, arguments) || this;
@@ -85,7 +90,7 @@ var LeadGuard = (function (_super) {
     ;
     return LeadGuard;
 }(RoyalGuard));
-var FollowerGuard = (function (_super) {
+var FollowerGuard = /** @class */ (function (_super) {
     __extends(FollowerGuard, _super);
     function FollowerGuard() {
         return _super !== null && _super.apply(this, arguments) || this;
