@@ -1,5 +1,4 @@
 //// [unusedPrivateVariableInClass4.ts]
-
 class greeter {
     private x: string;
     private y: string;
@@ -11,7 +10,7 @@ class greeter {
 }
 
 //// [unusedPrivateVariableInClass4.js]
-var greeter = (function () {
+var greeter = /** @class */ (function () {
     function greeter() {
     }
     greeter.prototype.method1 = function () {
