@@ -1,5 +1,4 @@
 //// [unusedPrivateVariableInClass5.ts]
-
 class greeter {
     private x: string;
     private y: string;
@@ -11,7 +10,7 @@ class greeter {
 }
 
 //// [unusedPrivateVariableInClass5.js]
-var greeter = (function () {
+var greeter = /** @class */ (function () {
     function greeter() {
         this.x = "dummy value";
     }
