@@ -7,7 +7,7 @@
 ////
 ////    }
 ////
-////    public /**/[|start|](){
+////    public /**/[|{| "isWriteAccess": true, "isDefinition": true |}start|](){
 ////        return this;
 ////    }
 ////
@@ -23,11 +23,21 @@
 ////second.[|start|]();
 ////second.stop();
 
-verify.rangesReferenceEachOther();
+checkRefs();
 
 cancellation.setCancelled();
-verifyOperationIsCancelled(() => verify.rangesReferenceEachOther());
+verifyOperationIsCancelled(checkRefs);
 
 // verify that internal state is still correct
 cancellation.resetCancelled();
-verify.rangesReferenceEachOther();
+checkRefs();
+
+function checkRefs() {
+    const ranges = test.ranges();
+    const [r0, r1] = ranges;
+    verify.referenceGroups(r0, [{ definition: "(method) Test.start(): this", ranges }]);
+    verify.referenceGroups(r1, [
+        { definition: "(method) Second.Test.start(): Second.Test", ranges: [r0] },
+        { definition: "(method) Second.Test.start(): Second.Test", ranges: [r1] }
+    ]);
+}
