@@ -163,11 +163,11 @@ verify.currentParameterHelpArgumentDocCommentIs("");
 verify.quickInfoAt("33q", "(method) i2.nc_fnfoo(b: number): string");
 
 goTo.marker('34');
-verify.completionListContains("i1", "interface i1", "this is interface 1");
+verify.not.completionListContains("i1", "interface i1", "this is interface 1");
 verify.completionListContains("i1_i", "var i1_i: i1", "");
-verify.completionListContains("nc_i1", "interface nc_i1", "");
+verify.not.completionListContains("nc_i1", "interface nc_i1", "");
 verify.completionListContains("nc_i1_i", "var nc_i1_i: nc_i1", "");
-verify.completionListContains("i2", "interface i2", "this is interface 2 with memebers");
+verify.not.completionListContains("i2", "interface i2", "this is interface 2 with memebers");
 verify.completionListContains("i2_i", "var i2_i: i2", "");
 verify.completionListContains("i2_i_x", "var i2_i_x: number", "");
 verify.completionListContains("i2_i_foo", "var i2_i_foo: (b: number) => string", "");
@@ -194,7 +194,7 @@ verify.completionListContains("a", "(parameter) a: number", "i3_i a");
 
 verify.quickInfoAt("40q", "var i3_i: i3");
 goTo.marker('40');
-verify.completionListContains("i3", "interface i3", "");
+verify.not.completionListContains("i3", "interface i3", "");
 verify.completionListContains("i3_i", "var i3_i: i3", "");
 
 goTo.marker('41');
