@@ -41,6 +41,14 @@
 ////class d extends a.C { constructor() { [|super|](); }
 
 const ranges = test.ranges();
-for (const ctr of ranges.slice(0, 3)) {
-    verify.referencesOf(ctr, ranges);
+const [a0, a1, a2, a3, a4, b0, c0, d0, d1] = ranges;
+verify.referenceGroups([a0, a2], defs("constructor C(n: number): C (+1 overload)"));
+verify.referenceGroups(a1, defs("constructor C(): C (+1 overload)"));
+
+function defs(definition: string) {
+    return [
+        { definition, ranges: [a0, a1, a2, a3, d0, d1, a4] },
+        { definition: "import C", ranges: [b0] },
+        { definition: "import C", ranges: [c0] }
+    ]
 }
