@@ -2,7 +2,7 @@
 
 // @Filename: a.ts
 ////interface A {
-////    [|foo|]: string;
+////    [|{| "isWriteAccess": true, "isDefinition": true |}foo|]: string;
 ////}
 
 // @Filename: b.ts
@@ -12,9 +12,9 @@
 ////    x.[|foo|]
 ////}
 
-verify.rangesReferenceEachOther();
+verify.singleReferenceGroup("(property) A.foo: string");
 
 goTo.marker("");
 edit.insert("\r\n");
 
-verify.rangesReferenceEachOther();
+verify.singleReferenceGroup("(property) A.foo: string");
