@@ -1,12 +1,12 @@
 /// <reference path='fourslash.ts' />
 
 ////class MethodOverload {
-////    /*staticMethodOverload1*/static /*staticMethodOverload1Name*/method();
-////    /*staticMethodOverload2*/static method(foo: string);
-////    /*staticMethodDefinition*/static method(foo?: any) { }
-////    /*instanceMethodOverload1*/public /*instanceMethodOverload1Name*/method(): any;
-////    /*instanceMethodOverload2*/public method(foo: string);
-/////*instanceMethodDefinition*/public method(foo?: any) { return "foo" }
+////    static /*staticMethodOverload1*/method();
+////    static /*staticMethodOverload2*/method(foo: string);
+////    static /*staticMethodDefinition*/method(foo?: any) { }
+////    public /*instanceMethodOverload1*/method(): any;
+////    public /*instanceMethodOverload2*/method(foo: string);
+////    public /*instanceMethodDefinition*/method(foo?: any) { return "foo" }
 ////}
 
 ////// static method
@@ -23,6 +23,6 @@ verify.goToDefinition({
     staticMethodReference2: "staticMethodOverload2",
     instanceMethodReference1: "instanceMethodOverload1",
     instanceMethodReference2: "instanceMethodOverload2",
-    staticMethodOverload1Name: "staticMethodDefinition",
-    instanceMethodOverload1Name: "instanceMethodDefinition"
+    staticMethodOverload1: "staticMethodDefinition",
+    instanceMethodOverload1: "instanceMethodDefinition"
 });
