@@ -5,7 +5,7 @@
 ////var n = 14;
 ////
 ////class foo {
-////    private [|n|] = 0;
+////    private [|{| "isWriteAccess": true, "isDefinition": true |}n|] = 0;
 ////
 ////    public bar() {
 ////        this.[|n|] = 9;
@@ -20,4 +20,4 @@
 ////    }
 ////}
 
-verify.rangesReferenceEachOther();
+verify.singleReferenceGroup("(property) foo.n: number");
