@@ -6,15 +6,17 @@
 ////     interface IntrinsicElements { }
 ////     interface ElementAttributesProperty { props; }
 //// }
-//// /*ct*/class MyClass {
+//// class /*ct*/MyClass {
 ////     props: {
 ////         /*pt*/foo: string;
 ////     }
 //// }
 //// var x = <My/*c*/Class />;
 //// var y = <MyClass f/*p*/oo= 'hello' />;
+//// var z = <MyCl/*w*/ass wrong= 'hello' />;
 
 verify.goToDefinition({
     c: "ct",
-    p: "pt"
+    p: "pt",
+    w: "ct"
 });
